/*****************************************************************************
* | File      	:   Touch_1IN69.c
* | Author      :   Waveshare team
* | Function    :   Hardware underlying interface
* | Info        :
*                Used to shield the underlying layers of each master
*                and enhance portability
*----------------
* |	This version:   V1.0
* | Date        :   2024-03-25
* | Info        :   Basic version
*
******************************************************************************/
#include "Touch_1in69.h"


/******************************************************************************
function :	读取ID
parameter:  CST816T : 0xB5
******************************************************************************/
UBYTE Touch_1IN69_WhoAmI()
{
    if (DEV_I2C_Read_Byte(address,0xA7) == 0xB5)   
        return true;
    else
        return false;
}

/******************************************************************************
function :	复位触摸
parameter: 
******************************************************************************/
void Touch_1IN69_Reset()
{
    DEV_Digital_Write(DEV_I2C_RST, 0);
    DEV_Delay_ms(100);
    DEV_Digital_Write(DEV_I2C_RST, 1);
    DEV_Delay_ms(100);
}

/******************************************************************************
function :	读取软件版本号
parameter:  
******************************************************************************/
UBYTE Touch_1IN69_Read_Revision()
{
    return DEV_I2C_Read_Byte(address,0xA9);
}

/******************************************************************************
function :	退出休眠模式
parameter:  
******************************************************************************/
void Touch_1IN69_Stop_Sleep()
{
    DEV_I2C_Write_Byte(address,DisAutoSleep,0x07);
}

/******************************************************************************
function :	设置触摸模式
parameter:  
        mode = 0 gestures mode 
        mode = 1 point mode
        mode = 2 mixed mode
******************************************************************************/
void Touch_1IN69_Set_Mode(UBYTE mode)
{
    if (mode == 1)
    {
        DEV_I2C_Write_Byte(address,IrqCtl,0X60);
        // DEV_I2C_Write_Byte(address,NorScanPer,0X01);//正常快速检测周期  单位10ms      
        // DEV_I2C_Write_Byte(address,IrqPluseWidth,0x01); //中断低脉冲输出宽度 1.5MS
    }       
    else if(mode == 2)
        DEV_I2C_Write_Byte(address,IrqCtl,0X71);
    else
        {
            DEV_I2C_Write_Byte(address,IrqCtl,0X11);
            DEV_I2C_Write_Byte(address,NorScanPer,0X01);
            DEV_I2C_Write_Byte(address,IrqPluseWidth,0x01);//中断低脉冲输出宽度 1.5MS
            DEV_I2C_Write_Byte(address,MotionMask,EnDClick);//使能双击模式
        }

}

/******************************************************************************
function :	唤醒触摸屏
parameter:  
******************************************************************************/
void Touch_1IN69_Wake_up()
{
    DEV_Digital_Write(DEV_I2C_RST, 0);
    DEV_Delay_ms(10);
    DEV_Digital_Write(DEV_I2C_RST, 1);
    DEV_Delay_ms(50);
    DEV_I2C_Write_Byte(address,0xFE,0x01);
}


/******************************************************************************
function :	屏幕初始化
parameter:  
******************************************************************************/
UBYTE Touch_1IN69_init(UBYTE mode)
{
    UBYTE bRet,Rev;
    Touch_1IN69_Reset();
    
    bRet = Touch_1IN69_WhoAmI();
    if (bRet)
    {
        printf("Success:Detected CST816D.\r\n");
        Rev = Touch_1IN69_Read_Revision();
        printf("CST816D Revision = %d\r\n",Rev);
        Touch_1IN69_Stop_Sleep();//disable_low_power
        // DEV_I2C_Write_Byte(address,0xe5,0x03); //enter_deep_sleep
        // DEV_I2C_Write_Byte(address,0xb5,0x03); //enter_pin_voltage_test
        // uint16_t value;
        // DEV_I2C_Write_Byte(address,0XFE,0x06);
        // DEV_I2C_Read_Register(address,0x40, &value);
        // printf("read_rawdata = %d\r\n",value);

        // DEV_I2C_Write_Byte(address,0XFE,0x07);
        // DEV_I2C_Read_Register(address,0x60, &value);
        // printf("read_diff = %d\r\n",value);
    }
    else
    {
        printf("Error: Not Detected CST816T.\r\n");
        return false;
    }
    // XY.mode = mode;
    Touch_1IN69_Set_Mode(mode);

    XY.x_point = 0;
    XY.y_point = 0;
    return true;
}

/******************************************************************************
function :	获取对应的点坐标
parameter:  
******************************************************************************/
Touch_1IN69_XY Touch_1IN69_Get_Point()
{
    UBYTE data[4];
    DEV_I2C_Read_nByte(address, 0x03, data, 4);
    
    XY.x_point = ((data[0] & 0x0f)<<8) + data[1];
    XY.y_point = ((data[2] & 0x0f)<<8) + data[3];
    
    return XY;
}












